#!perl -T

use Test::More tests => 1;

BEGIN {
	use_ok( 'Rose::DBx::Object::MoreHelpers' );
}

diag( "Testing Rose::DBx::Object::MoreHelpers $Rose::DBx::Object::MoreHelpers::VERSION, Perl $], $^X" );
